/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "Storage.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomStorage.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomStorage.h"
#include "gdom/PrivateIntGdomStorage.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Storage* obj)
{
    return WebCore::cacheDOMObject<WebCore::Storage, GDOMStorage>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMStorage::GDOMStorage(PassRefPtr<WebCore::Storage> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMStorage::~GDOMStorage()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

gchar *
gdom_storage_key (GdomStorage *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Storage * item = (gdom_storage_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->key(index, ec));
    return res;

}

gchar *
gdom_storage_get_item (GdomStorage *thiz, gchar * key)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Storage * item = (gdom_storage_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (key, 0);
    WebCore::String _g_key = WebCore::String::fromUTF8(key);
    gchar * res = _G_GET_UTF8(item->getItem(_g_key));
    return res;

}

void
gdom_storage_set_item (GdomStorage *thiz, gchar * key, gchar * data)
{
    g_return_if_fail (thiz);
    WebCore::Storage * item = (gdom_storage_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (key);
    g_return_if_fail (data);
    WebCore::String _g_key = WebCore::String::fromUTF8(key);
    WebCore::String _g_data = WebCore::String::fromUTF8(data);
    WebCore::ExceptionCode ec = 0;
    item->setItem(_g_key, _g_data, ec);

}

void
gdom_storage_remove_item (GdomStorage *thiz, gchar * key)
{
    g_return_if_fail (thiz);
    WebCore::Storage * item = (gdom_storage_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (key);
    WebCore::String _g_key = WebCore::String::fromUTF8(key);
    item->removeItem(_g_key);

}

void
gdom_storage_clear (GdomStorage *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Storage * item = (gdom_storage_get_private(thiz));
    g_return_if_fail (item);
    item->clear();

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Storage.h"
#include <gdom/GdomStorage.h>
#include <gdom/PrivateGdomStorage.h>
#include <gdom/PrivateIntGdomStorage.h>
#include <gdom/GdomStorageprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomStorage, gdom_storage, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomStoragePrivate {
    class WebCore::Storage *item;
};

#define GDOM_STORAGE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_STORAGE, GdomStoragePrivate))
*/
G_END_DECLS
WebCore::Storage *
gdom_storage_get_private(GdomStorage *request)
{
    GDOMStorage *res;
    res = static_cast<GDOMStorage*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Storage*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_LENGTH,
};


static void gdom_storage_finalize(GObject* object)
{
    /*GdomStorage* request = GDOM_STORAGE(object);*/
    /*GdomStoragePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_storage_parent_class)->finalize(object);
}

static void gdom_storage__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomStorage *self = GDOM_STORAGE(object);
    WebCore::Storage * gself = (gdom_storage_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_storage__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomStorage *self = GDOM_STORAGE(object);
    WebCore::Storage * gself = (gdom_storage_get_private(self));
    switch (prop_id) {
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_storage_class_init(GdomStorageClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_storage_finalize;
    g_klass->set_property = gdom_storage__set_property_fn;
    g_klass->get_property = gdom_storage__get_property_fn;

    g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "storage_length", /* short description */
                "read-only  gulong Storage.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomStoragePrivate));*/
}

static void gdom_storage_init(GdomStorage* request)
{
    /*GdomStoragePrivate* priv = GDOM_STORAGE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

